// Copyright (C) 2023 WYATT GROUP
// Please see the AUTHORS file for details.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.

import 'package:copy_with_extension/copy_with_extension.dart';
import 'package:flutter/material.dart';
import 'package:wyatt_ui_components/src/core/utils/component_registry.dart';
import 'package:wyatt_ui_components/src/domain/entities/entities.dart';
part 'component_theme_data.g.dart';

/// {@template component_theme_data}
/// A class that holds all the components that are used in the app.
/// {@endtemplate}
@CopyWith()
class ComponentThemeData {
  /// {@macro component_theme_data}
  factory ComponentThemeData({
    ComponentRegistry<TopAppBarComponent>? topAppBars,
    ComponentRegistry<TopNavigationBarComponent>? topNavigationBars,
    ComponentRegistry<BottomNavigationBarComponent>? bottomNavigationBars,
    ComponentRegistry<ErrorComponent>? errors,
    ComponentRegistry<LoaderComponent>? loaders,
    ComponentRegistry<RichTextBuilderComponent>? richTextBuilders,
    ComponentRegistry<TextInputComponent>? textInputs,
    ComponentRegistry<FileSelectionButtonComponent>? fileSelectionButtons,
    ComponentRegistry<FlatButtonComponent>? flatButtons,
    ComponentRegistry<SimpleIconButtonComponent>? simpleIconButtons,
    ComponentRegistry<SymbolButtonComponent>? symbolButtons,
    ComponentRegistry<InformationCardComponent>? informationCards,
    ComponentRegistry<PortfolioCardComponent>? portfolioCards,
    ComponentRegistry<QuoteCardComponent>? quoteCards,
    ComponentRegistry<SkillCardComponent>? skillCards,
    ComponentRegistry<PricingCardComponent>? pricingCards,
    ComponentRegistry<FloatingActionButtonComponent>? floatingActionButtons,
  }) =>
      ComponentThemeData.raw(
        topAppBars: topAppBars,
        topNavigationBars: topNavigationBars,
        bottomNavigationBars: bottomNavigationBars,
        errors: errors,
        loaders: loaders,
        richTextBuilders: richTextBuilders,
        textInputs: textInputs,
        fileSelectionButtons: fileSelectionButtons,
        flatButtons: flatButtons,
        simpleIconButtons: simpleIconButtons,
        symbolButtons: symbolButtons,
        informationCards: informationCards,
        portfolioCards: portfolioCards,
        quoteCards: quoteCards,
        skillCards: skillCards,
        pricingCards: pricingCards,
        floatingActionButtons: floatingActionButtons,
      );

  // /// {@macro component_theme_data}
  factory ComponentThemeData.fromOther(ComponentThemeData other) =>
      ComponentThemeData(
        topAppBars: other.topAppBars,
        topNavigationBars: other.topNavigationBars,
        bottomNavigationBars: other.bottomNavigationBars,
        errors: other.errors,
        loaders: other.loaders,
        richTextBuilders: other.richTextBuilders,
        textInputs: other.textInputs,
        fileSelectionButtons: other.fileSelectionButtons,
        flatButtons: other.flatButtons,
        simpleIconButtons: other.simpleIconButtons,
        symbolButtons: other.symbolButtons,
        informationCards: other.informationCards,
        portfolioCards: other.portfolioCards,
        quoteCards: other.quoteCards,
        skillCards: other.skillCards,
        pricingCards: other.pricingCards,
        floatingActionButtons: other.floatingActionButtons,
      );

  /// Create a [ComponentThemeData] given a set of exact values. Most values
  /// must be specified.
  ///
  /// This will rarely be used directly. It is used by lerp to
  /// create intermediate themes based on two themes created with the
  /// [ComponentThemeData] constructor.
  const ComponentThemeData.raw({
    this.topAppBars,
    this.topNavigationBars,
    this.bottomNavigationBars,
    this.errors,
    this.loaders,
    this.richTextBuilders,
    this.textInputs,
    this.fileSelectionButtons,
    this.flatButtons,
    this.simpleIconButtons,
    this.symbolButtons,
    this.informationCards,
    this.portfolioCards,
    this.quoteCards,
    this.skillCards,
    this.pricingCards,
    this.floatingActionButtons,
  });

  R _get<T extends Component, R>(T? component, R? returned) {
    if (component == null) {
      throw FlutterError('No $T component provided.\n'
          'Please provide a $T component in the ComponentThemeData.');
    }

    if (returned == null) {
      throw FlutterError('$T does not have a CopyWith method.\n'
          'Please provide a $T component that implements CopyWith '
          'in the ComponentThemeData.');
    }

    return returned;
  }

  // Bars
  final ComponentRegistry<TopAppBarComponent>? topAppBars;
  $TopAppBarComponentCWProxy topAppBarComponent([String? id]) =>
      _get(topAppBars?.call(id), topAppBars?.call(id)?.copyWith);

  final ComponentRegistry<TopNavigationBarComponent>? topNavigationBars;
  $TopNavigationBarComponentCWProxy topNavigationBarComponent([String? id]) =>
      _get(
        topNavigationBars?.call(id),
        topNavigationBars?.call(id)?.copyWith,
      );

  final ComponentRegistry<BottomNavigationBarComponent>? bottomNavigationBars;
  $BottomNavigationBarComponentCWProxy bottomNavigationBarComponent([
    String? id,
  ]) =>
      _get(
        bottomNavigationBars?.call(id),
        bottomNavigationBars?.call(id)?.copyWith,
      );

  // CRUD Widgets
  final ComponentRegistry<ErrorComponent>? errors;
  $ErrorComponentCWProxy errorComponent([String? id]) =>
      _get(errors?.call(id), errors?.call(id)?.copyWith);

  final ComponentRegistry<LoaderComponent>? loaders;
  $LoaderComponentCWProxy loaderComponent([String? id]) =>
      _get(loaders?.call(id), loaders?.call(id)?.copyWith);

  // Cards
  final ComponentRegistry<InformationCardComponent>? informationCards;
  $InformationCardComponentCWProxy informationCardComponent([String? id]) =>
      _get(informationCards?.call(id), informationCards?.call(id)?.copyWith);

  final ComponentRegistry<PortfolioCardComponent>? portfolioCards;
  $PortfolioCardComponentCWProxy portfolioCardComponent([String? id]) =>
      _get(portfolioCards?.call(id), portfolioCards?.call(id)?.copyWith);

  final ComponentRegistry<QuoteCardComponent>? quoteCards;
  $QuoteCardComponentCWProxy quoteCardComponent([String? id]) =>
      _get(quoteCards?.call(id), quoteCards?.call(id)?.copyWith);

  final ComponentRegistry<SkillCardComponent>? skillCards;
  $SkillCardComponentCWProxy skillCardComponent([String? id]) =>
      _get(skillCards?.call(id), skillCards?.call(id)?.copyWith);

  final ComponentRegistry<PricingCardComponent>? pricingCards;
  $PricingCardComponentCWProxy pricingCardComponent([String? id]) =>
      _get(pricingCards?.call(id), pricingCards?.call(id)?.copyWith);

  // Rich Text
  final ComponentRegistry<RichTextBuilderComponent>? richTextBuilders;
  $RichTextBuilderComponentCWProxy richTextBuilderComponent([String? id]) =>
      _get(richTextBuilders?.call(id), richTextBuilders?.call(id)?.copyWith);

  // Text Inputs
  final ComponentRegistry<TextInputComponent>? textInputs;
  $TextInputComponentCWProxy textInputComponent([String? id]) =>
      _get(textInputs?.call(id), textInputs?.call(id)?.copyWith);

  // Buttons
  final ComponentRegistry<FileSelectionButtonComponent>? fileSelectionButtons;
  $FileSelectionButtonComponentCWProxy fileSelectionButtonComponent([
    String? id,
  ]) =>
      _get(
        fileSelectionButtons?.call(id),
        fileSelectionButtons?.call(id)?.copyWith,
      );

  final ComponentRegistry<FlatButtonComponent>? flatButtons;
  $FlatButtonComponentCWProxy flatButtonComponent([String? id]) =>
      _get(flatButtons?.call(id), flatButtons?.call(id)?.copyWith);

  final ComponentRegistry<SimpleIconButtonComponent>? simpleIconButtons;
  $SimpleIconButtonComponentCWProxy simpleIconButtonComponent([String? id]) =>
      _get(
        simpleIconButtons?.call(id),
        simpleIconButtons?.call(id)?.copyWith,
      );

  final ComponentRegistry<SymbolButtonComponent>? symbolButtons;
  $SymbolButtonComponentCWProxy symbolButtonComponent([String? id]) =>
      _get(symbolButtons?.call(id), symbolButtons?.call(id)?.copyWith);

  final ComponentRegistry<FloatingActionButtonComponent>? floatingActionButtons;
  $FloatingActionButtonComponentCWProxy floatingActionButtonComponent([
    String? id,
  ]) =>
      _get(
        floatingActionButtons?.call(id),
        floatingActionButtons?.call(id)?.copyWith,
      );
}
